import { TracerInterface } from '.';
import { HandlerMethodDecorator, TracerOptions, MethodDecorator } from './types';
import { ProviderServiceInterface } from './provider';
import { Segment, Subsegment } from 'aws-xray-sdk-core';
/**
 * ## Intro
 * Tracer is an opinionated thin wrapper for [AWS X-Ray SDK for Node.js](https://github.com/aws/aws-xray-sdk-node).
 *
 * Tracing data can be visualized through AWS X-Ray Console.
 *
 * ## Key features
 *   * Auto capture cold start as annotation, and responses or full exceptions as metadata
 *   * Auto-disable when not running in AWS Lambda environment
 *   * Support tracing functions via decorators, middleware, and manual instrumentation
 *   * Support tracing AWS SDK v2 and v3 via AWS X-Ray SDK for Node.js
 *
 * ## Usage
 *
 * ### Functions usage with middlewares
 *
 * If you use function-based Lambda handlers you can use the [captureLambdaHanlder()](./_aws_lambda_powertools_tracer.Tracer.html) middy middleware to automatically:
 * * handle the subsegment lifecycle
 * * add the `ColdStart` annotation
 * * add the function response as metadata
 * * add the function error as metadata (if any)
 *
 * @example
 * ```typescript
 * import { Tracer, captureLambdaHandler } from '@aws-lambda-powertools/tracer';
 * import middy from '@middy/core';
 *
 * const tracer = new Tracer({ serviceName: 'my-service' });
 *
 * export const handler = middy(async (_event: any, _context: any) => {
 *   ...
 * }).use(captureLambdaHandler(tracer));
 * ```
 *
 * ### Object oriented usage with decorators
 *
 * If instead you use TypeScript Classes to wrap your Lambda handler you can use the [@tracer.captureLambdaHanlder()](./_aws_lambda_powertools_tracer.Tracer.html#captureLambdaHanlder) decorator to automatically:
 * * handle the subsegment lifecycle
 * * add the `ColdStart` annotation
 * * add the function response as metadata
 * * add the function error as metadata (if any)
 *
 * @example
 * ```typescript
 * import { Tracer } from '@aws-lambda-powertools/tracer';
 *
 * const tracer = new Tracer({ serviceName: 'my-service' });
 *
 * // FYI: Decorator might not render properly in VSCode mouse over due to https://github.com/microsoft/TypeScript/issues/39371 and might show as *@tracer* instead of `@tracer.captureLambdaHanlder`
 *
 * class Lambda {
 *   @tracer.captureLambdaHanlder()
 *   public handler(event: any, context: any) {
 *     ...
 *   }
 * }
 *
 * export const handlerClass = new Lambda();
 * export const handler = handlerClass.handler;
 * ```
 *
 * ### Functions usage with manual instrumentation
 *
 * If you prefer to manually instrument your Lambda handler you can use the methods in the tracer class directly.
 *
 * @example
 * ```typescript
 * import { Tracer } from '@aws-lambda-powertools/tracer';
 * import { Segment } from 'aws-xray-sdk-core';
 *
 * const tracer = new Tracer({ serviceName: 'my-service' });
 *
 * export const handler = async (_event: any, context: any) => {
 *   // Create subsegment & set it as active
 *   const subsegment = new Subsegment(`## ${context.functionName}`);
 *   tracer.setSegment(subsegment);
 *   // Add the ColdStart annotation
 *   this.putAnnotation('ColdStart', tracer.isColdStart());
 *
 *   let res;
 *   try {
 *     res = await someLogic(); // Do something
 *     // Add the response as metadata
 *     tracer.putMetadata(`${context.functionName} response`, data);
 *   } catch (err) {
 *     // Add the error as metadata
 *     subsegment.addError(err, false);
 *   }
 *
 *   // Close subsegment
 *   subsegment.close();
 *
 *   return res;
 * }
  * ```
 */
declare class Tracer implements TracerInterface {
    static coldStart: boolean;
    provider: ProviderServiceInterface;
    private captureError;
    private captureResponse;
    private customConfigService?;
    private envVarsService?;
    private serviceName;
    private tracingEnabled;
    constructor(options?: TracerOptions);
    /**
     * Patch all AWS SDK v2 clients and create traces when your application makes calls to AWS services.
     *
     * If you want to patch a specific client use {@link captureAWSClient} and if you are using AWS SDK v3 use {@link captureAWSv3Client} instead.
     *
     * @see https://docs.aws.amazon.com/xray/latest/devguide/xray-sdk-nodejs-awssdkclients.html
     *
     * @example
     * ```typescript
     * import { Tracer } from '@aws-lambda-powertools/tracer';
     *
     * const tracer = new Tracer({ serviceName: 'my-service' });
     * const AWS = tracer.captureAWS(require('aws-sdk'));
     *
     * export const handler = async (_event: any, _context: any) => {
     *   ...
     * }
     * ```
     *
     * @param aws - AWS SDK v2 import
     * @returns AWS - Instrumented AWS SDK
     */
    captureAWS<T>(aws: T): T;
    /**
     * Patch a specific AWS SDK v2 client and create traces when your application makes calls to that AWS service.
     *
     * If you want to patch all clients use {@link captureAWS} and if you are using AWS SDK v3 use {@link captureAWSv3Client} instead.
     *
     * @see https://docs.aws.amazon.com/xray/latest/devguide/xray-sdk-nodejs-awssdkclients.html
     *
     * @example
     * ```typescript
     * import { S3 } from "aws-sdk";
     * import { Tracer } from '@aws-lambda-powertools/tracer';
     *
     * const tracer = new Tracer({ serviceName: 'my-service' });
     * tracer.captureAWS(require('aws-sdk'));
     * const s3 = tracer.captureAWSClient(new S3({ apiVersion: "2006-03-01" }));
     *
     * export const handler = async (_event: any, _context: any) => {
     *   ...
     * }
     * ```
     *
     * @param service - AWS SDK v2 client
     * @returns service - Instrumented AWS SDK v2 client
     */
    captureAWSClient<T>(service: T): T;
    /**
     * Patch an AWS SDK v3 client and create traces when your application makes calls to that AWS service.
     *
     * If you are using AWS SDK v2 use {@link captureAWSClient} instead.
     *
     * @see https://docs.aws.amazon.com/xray/latest/devguide/xray-sdk-nodejs-awssdkclients.html
     *
     * @example
     * ```typescript
     * import { S3Client } from "@aws-sdk/client-s3";
     * import { Tracer } from '@aws-lambda-powertools/tracer';
     *
     * const tracer = new Tracer({ serviceName: 'my-service' });
     * const client = new S3Client({});
     * tracer.captureAWSv3Client(client);
     *
     * export const handler = async (_event: any, _context: any) => {
     *   ...
     * }
     * ```
     *
     * @param service - AWS SDK v3 client
     * @returns service - Instrumented AWS SDK v3 client
     */
    captureAWSv3Client<T>(service: T): T;
    /**
     * A decorator automating capture of metadata and annotations on segments or subsegments for a Lambda Handler.
     *
     * Using this decorator on your handler function will automatically:
     * * handle the subsegment lifecycle
     * * add the `ColdStart` annotation
     * * add the function response as metadata
     * * add the function error as metadata (if any)
     *
     * Note: Currently TypeScript only supports decorators on classes and methods. If you are using the
     * function syntax, you should use the middleware instead.
     *
     * @example
     * ```typescript
     * import { Tracer } from '@aws-lambda-powertools/tracer';
     *
     * const tracer = new Tracer({ serviceName: 'my-service' });
     *
     * class Lambda {
     *   @tracer.captureLambdaHanlder()
     *   public handler(event: any, context: any) {
     *     ...
     *   }
     * }
     *
     * export const handlerClass = new Lambda();
     * export const handler = handlerClass.handler;
     * ```
     *
     * @decorator Class
     */
    captureLambdaHanlder(): HandlerMethodDecorator;
    /**
     * A decorator automating capture of metadata and annotations on segments or subsegments for an arbitrary function.
     *
     * Using this decorator on your function will automatically:
     * * handle the subsegment lifecycle
     * * add the function response as metadata
     * * add the function error as metadata (if any)
     *
     * Note: Currently TypeScript only supports decorators on classes and methods. If you are using the
     * function syntax, you should use the middleware instead.
     *
     * @example
     * ```typescript
     * import { Tracer } from '@aws-lambda-powertools/tracer';
     *
     * const tracer = new Tracer({ serviceName: 'my-service' });
     *
     * class Lambda {
     *   @tracer.captureMethod()
     *   public myMethod(param: any) {
     *     ...
     *   }
     *
     *   public handler(event: any, context: any) {
     *     ...
     *   }
     * }
     *
     * export const handlerClass = new Lambda();
     * export const myMethod = handlerClass.myMethod;
     * export const handler = handlerClass.handler;
     * ```
     *
     * @decorator Class
     */
    captureMethod(): MethodDecorator;
    /**
     * Get the active segment or subsegment in the current scope.
     *
     * Usually you won't need to call this method unless you are manipulating segments using the escape hatch pattern.
     *
     * @see https://docs.aws.amazon.com/xray/latest/devguide/xray-concepts.html#xray-concepts-segments
     * @see https://awslabs.github.io/aws-lambda-powertools-typescript/latest/core/tracer/#escape-hatch-mechanism
     *
     * @example
     * ```typescript
     * import { Tracer } from '@aws-lambda-powertools/tracer';
     *
     * const tracer = new Tracer({ serviceName: 'my-service' });
     *
     * export const handler = async (_event: any, _context: any) => {
     *   const currentSegment = tracer.getSegment();
     *   ... // Do something with segment
     * }
     * ```
     *
     * @returns segment - The active segment or subsegment in the current scope.
     */
    getSegment(): Segment | Subsegment;
    /**
     * Get the current value of the `captureError` property.
     *
     * You can use this method during manual instrumentation to determine
     * if tracer should be capturing errors.
     *
     * @returns captureError - `true` if errors should be captured, `false` otherwise.
     */
    isCaptureErrorEnabled(): boolean;
    /**
     * Get the current value of the `captureResponse` property.
     *
     * You can use this method during manual instrumentation to determine
     * if tracer should be capturing function responses.
     *
     * @returns captureResponse - `true` if responses should be captured, `false` otherwise.
     */
    isCaptureResponseEnabled(): boolean;
    /**
     * Retrieve the current value of `ColdStart`.
     *
     * If Tracer has been initialized outside of the Lambda handler then the same instance
     * of Tracer will be reused throghout the lifecycle of that same Lambda execution environment
     * and this method will return `false` after the first invocation.
     *
     * @see https://docs.aws.amazon.com/lambda/latest/dg/runtimes-context.html
     *
     * @returns boolean - `true` if is cold start, otherwise `false`
     */
    static isColdStart(): boolean;
    /**
     * Get the current value of the `tracingEnabled` property.
     *
     * You can use this method during manual instrumentation to determine
     * if tracer is currently enabled.
     *
     * @returns tracingEnabled - `true` if tracing is enabled, `false` otherwise.
     */
    isTracingEnabled(): boolean;
    /**
     * Adds annotation to existing segment or subsegment.
     *
     * @see https://docs.aws.amazon.com/xray/latest/devguide/xray-sdk-nodejs-segment.html#xray-sdk-nodejs-segment-annotations
     *
     * @example
     * ```typescript
     * import { Tracer } from '@aws-lambda-powertools/tracer';
     *
     * const tracer = new Tracer({ serviceName: 'my-service' });
     *
     * export const handler = async (_event: any, _context: any) => {
     *   tracer.putAnnotation('PaymentStatus', "SUCCESS");
     * }
     * ```
     *
     * @param key - Annotation key
     * @param value - Value for annotation
     */
    putAnnotation(key: string, value: string | number | boolean): void;
    /**
     * Adds metadata to existing segment or subsegment.
     *
     * @see https://docs.aws.amazon.com/xray/latest/devguide/xray-sdk-nodejs-segment.html#xray-sdk-nodejs-segment-metadata
     *
     * @example
     * ```typescript
     * import { Tracer } from '@aws-lambda-powertools/tracer';
     *
     * const tracer = new Tracer({ serviceName: 'my-service' });
     *
     * export const handler = async (_event: any, _context: any) => {
     *   const res = someLogic();
     *   tracer.putMetadata('PaymentResponse', res);
     * }
     * ```
     *
     * @param key - Metadata key
     * @param value - Value for metadata
     * @param timestamp - Namespace that metadata will lie under, if none is passed it will use the serviceName
     */
    putMetadata(key: string, value: unknown, namespace?: string | undefined): void;
    /**
     * Sets the passed subsegment as the current active subsegment.
     *
     * If you are using a middleware or a decorator this is done automatically for you.
     *
     * @see https://docs.aws.amazon.com/xray/latest/devguide/xray-sdk-nodejs-subsegments.html
     *
     * @example
     * ```typescript
     * import { Tracer } from '@aws-lambda-powertools/tracer';
     * import { Segment } from 'aws-xray-sdk-core';
     *
     * const tracer = new Tracer({ serviceName: 'my-service' });
     *
     * export const handler = async (_event: any, _context: any) => {
     *   const subsegment = new Subsegment('### foo.bar');
     *   tracer.setSegment(subsegment);
     * }
     * ```
     *
     * @param segment - Subsegment to set as the current segment
     */
    setSegment(segment: Segment | Subsegment): void;
    /**
      * Add an error to the current segment or subsegment as metadata.
      * Used internally by decoratorators and middlewares.
      *
      * @see https://docs.aws.amazon.com/xray/latest/devguide/xray-concepts.html#xray-concepts-errors
      *
      * @param error - Error to serialize as metadata
      */
    private addErrorAsMetadata;
    /**
      * Add an data to the current segment or subsegment as metadata.
      * Used internally by decoratorators and middlewares.
      *
      * @see https://docs.aws.amazon.com/xray/latest/devguide/xray-concepts.html#xray-concepts-errors
      *
      * @param data - Data to serialize as metadata
      * @param methodName - Name of the method that is being traced
      */
    private addResponseAsMetadata;
    /**
     * Add ColdStart annotation to the current segment or subsegment.
     * Used internally by decoratorators and middlewares.
     */
    private annotateColdStart;
    /**
     * Getter for `customConfigService`.
     * Used internally during initialization.
     */
    private getCustomConfigService;
    /**
     * Getter for `envVarsService`.
     * Used internally during initialization.
     */
    private getEnvVarsService;
    /**
     * Determine if we are running in a Lambda execution environment.
     * Used internally during initialization.
     */
    private isLambdaExecutionEnv;
    /**
     * Determine if we are running inside a SAM CLI process.
     * Used internally during initialization.
     */
    private isLambdaSamCli;
    /**
     * Validate that the service name provided is valid.
     * Used internally during initialization.
     *
     * @param serviceName - Service name to validate
     */
    private isValidServiceName;
    /**
     * Setter for `captureError` based on configuration passed and environment variables.
     * Used internally during initialization.
     */
    private setCaptureError;
    /**
     * Setter for `captureResponse` based on configuration passed and environment variables.
     * Used internally during initialization.
     */
    private setCaptureResponse;
    /**
     * Setter for `customConfigService` based on configuration passed.
     * Used internally during initialization.
     *
     * @param customConfigService - Custom configuration service to use
     */
    private setCustomConfigService;
    /**
     * Setter and initializer for `envVarsService`.
     * Used internally during initialization.
     */
    private setEnvVarsService;
    /**
     * Method that reconciles the configuration passed with the environment variables.
     * Used internally during initialization.
     *
     * @param options - Configuration passed to the tracer
     */
    private setOptions;
    /**
     * Setter for `customConfigService` based on configurations passed and environment variables.
     * Used internally during initialization.
     *
     * @param serviceName - Name of the service to use
     */
    private setServiceName;
    /**
     * Setter for `tracingEnabled` based on configurations passed and environment variables.
     * Used internally during initialization.
     *
     * @param enabled - Whether or not tracing is enabled
     */
    private setTracingEnabled;
}
export { Tracer };
//# sourceMappingURL=Tracer.d.ts.map